
#include <sys/ioctl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>

#include <linux/dvb/audio.h>

#include "mpegaudioheader.h"
#include "mpegpesheader.h"

#define MPEG_AUDIO_BUFF_SIZE 1024
#define MPEG_BASE_CLOCK_RATE 90000

void ElementaryStream ( FILE * inf, int afd )
{
	mpeg_audio_header_t frame_hdr;
	PES_header_t PES_hdr;
	int iFrameSize;
	unsigned char buff[MPEG_AUDIO_BUFF_SIZE];
	unsigned long ulPos;
	int iBytesRead;
	long lFilePos;
	long lPercentRead;
	
	struct stat statbuff;
	
	fstat ( fileno(inf), &statbuff );

	PES_hdr.stream_type = 0xc0;
	PES_hdr.PES_scrambling_control = 0;
	PES_hdr.PES_priority = 0;
	PES_hdr.data_alignment_indicator = 1;
	PES_hdr.copyright = 0;
	PES_hdr.original_or_copy = 0;
	PES_hdr.PTS = 0;

	while (mpeg_audio_header_find(inf,&frame_hdr))
	{
		iFrameSize = mpeg_audio_header_frame_size ( &frame_hdr );
						
		if (iFrameSize>=MPEG_AUDIO_HEADER_SIZE)
		{
			PES_hdr.PES_packet_length = (unsigned short) (MINIMUM_PES_PACKET_LENGTH + PTS_ENCODED_SIZE_IN_BYTES + iFrameSize);
			
			ulPos = pes_header_write (buff,0,&PES_hdr);
			
			write ( afd, buff, ulPos );
			write ( afd, &frame_hdr.headerData[0], MPEG_AUDIO_HEADER_SIZE);

			iBytesRead = fread ( buff, 1, iFrameSize-MPEG_AUDIO_HEADER_SIZE, inf );			
			if (iBytesRead>0)
			{
				write ( afd, buff, iBytesRead );				
			}
			
			PES_hdr.PTS += mpeg_audio_header_duration ( &frame_hdr, MPEG_BASE_CLOCK_RATE );
		}
		
		
		lFilePos = ftell ( inf );
		if (statbuff.st_size)
		{
			lPercentRead = (100 * lFilePos) / ((long) statbuff.st_size);
			
			printf ( "\r%ld%%.", lPercentRead );
			fflush ( stdout );
		}
		
	}
	
	printf ("\n\n");
}





void PacketisedElementaryStream ( FILE * inf, int afd )
{
	unsigned char buff[MPEG_AUDIO_BUFF_SIZE];
	int iResult;

	while (fread (buff,1,MPEG_AUDIO_BUFF_SIZE,inf)==MPEG_AUDIO_BUFF_SIZE)
	{
		iResult = write (afd,buff,MPEG_AUDIO_BUFF_SIZE);
	}
}



int main(int argc, char * argv[])
{
	if (argc<3)
	{
		printf ("Usage: elgar -[e|p] filename [volume]\n");
	}
	else
	{
		int afd;
		char *audiodev = "/dev/dvb/adapter0/audio0";
		
		if (getenv("AUDIO"))
			audiodev = getenv("AUDIO");
		
		printf("using audio device '%s'\n", audiodev);
		
		/* afd = open(audiodev, O_RDWR | O_NONBLOCK); */
		afd = open(audiodev, O_RDWR);
		if (afd < 0) 
		{
			perror("open audio device");
		}
		else
		{
		
			audio_status_t status;
			unsigned int uiCaps = 0;
		
			if (ioctl(afd, AUDIO_SELECT_SOURCE, AUDIO_SOURCE_MEMORY) == -1)
			{
				perror("AUDIO_SELECT_SOURCE");
			}
			if (ioctl(afd, AUDIO_SET_AV_SYNC, 0) == -1)
			{
				perror("AUDIO_SET_AV_SYNC");
			}
			if (ioctl(afd, AUDIO_PLAY) == -1)
			{
				perror("AUDIO_PLAY");
			}
			if (ioctl(afd, AUDIO_GET_STATUS, &status) == -1) 
			{
				perror("AUDIO_GET_STATUS");
			}
			if (ioctl(afd, AUDIO_GET_CAPABILITIES, &uiCaps) == -1) 
			{
				perror("AUDIO_GET_CAPABILITIES");
			}
			
			printf ( "Capabilities: %08x.\n", uiCaps );

			/*
			printf ( "Volume l: %u r: %u.\n",
				status.mixer_state.volume_left,status.mixer_state.volume_right);
			*/

			if (argc==4)
			{
				status.mixer_state.volume_left = (unsigned int) atoi(argv[3]);
				status.mixer_state.volume_right = (unsigned int) atoi(argv[3]);
	
				if (ioctl(afd, AUDIO_SET_MIXER, &status.mixer_state) == -1) 
				{
					perror("AUDIO_SET_MIXER");
				}
			}

			
			if (1)
			{
			
				FILE * inf = fopen (argv[2], "rb");
				
				if (inf)
				{
					if (argv[1][1] == 'e')
					{
						ElementaryStream ( inf, afd );
					}
					else if (argv[1][1] == 'p')
					{
						PacketisedElementaryStream ( inf, afd );
					}
					else
					{
						printf ("Usage: elgar -[e|p] filename [volume]\n");
					}
				
					fclose (inf);
				}
				else
				{
					perror("open audio file");
				}
			}
			
			close(afd);
			
		}
	}
	
	return 0;
}